--desc: selectively finds out of date statistics that did not auto-update
IF OBJECT_ID('tempdb..[#Results]') IS NOT NULL 
DROP TABLE [#Results] 
GO
     DECLARE @RowCount INT;
      --#################################################################################################
      --The Analysis: my crappy assumptions:
      --UPDATE STATISTICS dbo.GMPVDET 
      --tables under 1000 rows, I'll leave then at 20%
      --tables with more rows than that, I'll use an arbitrary sliding scale formula.
      --formula to be modified  based on analysis
      --#################################################################################################
   PRINT '--#################################################################################################';
      PRINT '  EXECUTING sp_UpdateStatistics in Database ' + QUOTENAME(DB_NAME());
      SELECT
     ROW_NUMBER() OVER(PARTITION BY [X].[SCHEMA Name],[X].[TABLE Name] ORDER BY [X].[SCHEMA Name],[X].[TABLE Name],X.[ROWS Modified] DESC) AS [RW],
        [X].*,
        ISNULL(CASE
                 WHEN [X].[Total ROWS] <= 1000
                 THEN
                   CASE
                     WHEN [X].[PERCENT Modified] >= 20.0
                     THEN
                       'UPDATE STATISTICS ' + QUOTENAME([X].[SCHEMA Name]) + '.'
                       + QUOTENAME([X].[TABLE Name])
                       + ' WITH ALL, SAMPLE  --20% Small Table Rule'
        + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                   END
                 WHEN [X].[PERCENT Modified] = 100.00
                 THEN
                   'UPDATE STATISTICS ' + QUOTENAME([X].[SCHEMA Name]) + '.'
                   + QUOTENAME([X].[TABLE Name])
                   + ' WITH ALL, SAMPLE  --100% No real Stats Rule'
       + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                 --WHEN X.[Rows Modified] > 1000
                 --THEN 'UPDATE STATISTICS ' + [Schema Name] + '.' + [Table Name]     + ' WITH ALL, SAMPLE  --1000 Rows Modified Rule'
                 ELSE
                   CASE
                     WHEN [X].[Total ROWS] > 1000000000 --billion rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 0.1
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 1B Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                     WHEN [X].[Total ROWS] > 100000000 --hundred million rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 1.0
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 100M Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                     WHEN [X].[Total ROWS] > 10000000 --ten million rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 2.0
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 10M Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                     WHEN [X].[Total ROWS] > 1000000 --million rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 5.0
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 1M Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                     WHEN [X].[Total ROWS] > 100000 --hundred thousand rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 10.0
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 100K Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                     WHEN [X].[Total ROWS] > 10000 --ten thousand rows
                     THEN
                       CASE
                         WHEN [X].[PERCENT Modified] > 20.0
                         THEN
                           'UPDATE STATISTICS ' +  QUOTENAME([X].[SCHEMA Name]) + '.'
                           + QUOTENAME([X].[TABLE Name])
                           + ' WITH ALL, SAMPLE  -- 10K Big Table Rule'
         + ' | ' + CONVERT(VARCHAR,[X].[Total ROWS]) + ' Rows, ' + CONVERT(VARCHAR,[X].[PERCENT Modified]) + ' % Modified'
                       END
                   END
               END, '')  AS [STATISTICS SQL]
      INTO
        [#Results]
      FROM   (SELECT   DISTINCT 
      DB_NAME()   AS [DATABASE],
        s.name AS [SCHEMA Name],
        t.name AS [TABLE Name],
        --a.name,
        --sp.last_updated,
        sp.modification_counter  AS [ROWS Modified],
        sp.rows AS [Total ROWS],
        CASE
              WHEN sp.[modification_counter] > [sp].[rows]
              THEN
                100
              ELSE
                CONVERT(DECIMAL(8, 2), ( (  sp.[modification_counter] * 1.0 ) / [sp].[rows] * 1. ) * 100.0)
            END         AS [PERCENT Modified]
  FROM  sys.schemas s
        join sys.tables t on s.schema_id = t.schema_id
        join sys.stats a on t.object_id = a.object_id
        --JOIN sys.indexes i ON t.[object_id] = i.[object_id]
        CROSS APPLY sys.dm_db_stats_properties (t.object_id, a.stats_id) sp
WHERE sp.modification_counter > 1000) [X]
      WHERE  [X].[ROWS Modified] > 1000
      ORDER  BY
        [X].[ROWS Modified] DESC;
   --#################################################################################################
   -- Recap
   --#################################################################################################
      SELECT 
     @RowCount = COUNT(*)
      FROM   [#Results]
   WHERE  [STATISTICS SQL] <> ''
        AND [RW] = 1;
   PRINT '  sp_UpdateStatistics Recommended  [' + CONVERT(VARCHAR,@RowCount) + '] Tables to have Statistics Updated.'; 

      SELECT 
    [RW],[DATABASE],[SCHEMA Name],[TABLE Name],[ROWS Modified],[Total ROWS],[PERCENT Modified],
    'RAISERROR(''' + [SCHEMA Name] + '.' + [TABLE Name] +  ' Item ' + CONVERT(VARCHAR(30),ROW_NUMBER() OVER(ORDER BY [SCHEMA Name],[TABLE Name],[ROWS Modified] DESC))  + ''',0,1)' + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10)
        + [STATISTICS SQL] + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10)  AS  [fSTATISTICS SQL]
      FROM   [#Results]
   WHERE  [STATISTICS SQL] <> ''
        AND [RW] = 1;