--desc: Captures missing index data and builds a CREATE INDEX command, with comment of performance impact for reference, now with index counts
--2025-03-22 adding current index usage and formatting commas in comments and output for readability

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
DECLARE @TableNameToCheck VARCHAR(128) = ''  --pasting [TableName] is now fine, code cleans it up for object name
SET @TableNameToCheck = REPLACE(REPLACE(@TableNameToCheck,']',''),'[','')

IF OBJECT_ID('tempdb..[#IndexSizes]') IS NOT NULL 
DROP TABLE [#IndexSizes] 

CREATE TABLE [#IndexSizes] ( 
[DatabaseName]             SYSNAME                                           NOT NULL,
[QualifiedObjectName]      VARCHAR(260)                                          NULL,
[equality_columns]         NVARCHAR(4000)                                        NULL,
[inequality_columns]       NVARCHAR(4000)                                        NULL,
[included_columns]         NVARCHAR(4000)                                        NULL,
[Approximate_IndexSizeGB]  NUMERIC(31,11)                                        NULL)

IF OBJECT_ID('tempdb..#ColumnSizes') IS NOT NULL DROP TABLE #ColumnSizes;
CREATE TABLE #ColumnSizes (
    DatabaseName SYSNAME,
    SchemaName SYSNAME,
    ObjectName SYSNAME,
    QualifiedObjectName varchar(260),
    ColumnName SYSNAME,
    ApproxColumnSize INT
);

IF OBJECT_ID('tempdb..[#TableCounts]') IS NOT NULL 
DROP TABLE [#TableCounts] 
CREATE TABLE [#TableCounts] ( 
[DatabaseName]         VARCHAR(128)                                NULL,
[QualifiedObjectName]  NVARCHAR(517)                               NULL,
[TheCount]             BIGINT                                      NULL)

IF OBJECT_ID('tempdb..[#TempIndexes]') IS NOT NULL 
DROP TABLE [#TempIndexes] 
CREATE TABLE [#TempIndexes] ( 
[DbObjectName]  VARCHAR(390)                                NULL,
[IndexCount]    INT                                         NULL)

execute sp_MSforeachdb '
INSERT INTO #ColumnSizes (DatabaseName, SchemaName,ObjectName, QualifiedObjectName,ColumnName, ApproxColumnSize)
SELECT 
quotename(''?''),
s.name,
o.name,
quotename(s.name) + ''.'' + quotename(o.name),
c.name,
CASE 
WHEN t.name IN (''tinyint'') THEN 1
WHEN t.name IN (''smallint'') THEN 2
WHEN t.name IN (''int'') THEN 4
WHEN t.name IN (''bigint'') THEN 8
WHEN t.name IN (''bit'') THEN 1
WHEN t.name IN (''datetime'', ''smalldatetime'') THEN 8
WHEN t.name IN (''money'', ''smallmoney'') THEN 8
WHEN t.name IN (''float'') THEN 8
WHEN t.name IN (''real'') THEN 4
WHEN t.name IN (''uniqueidentifier'') THEN 16
WHEN t.name IN (''nvarchar'', ''nchar'') THEN c.max_length / 2
WHEN t.name IN (''varchar'', ''char'') THEN c.max_length
ELSE 100
END
FROM [?].sys.columns c
JOIN [?].sys.objects o ON c.object_id = o.object_id
JOIN [?].sys.schemas s ON s.schema_id = o.schema_id
JOIN [?].sys.types t ON c.user_type_id = t.user_type_id
WHERE o.type = ''U'';
'
execute sp_MSforeachdb '
INSERT INTO #TempIndexes([DbObjectName],[IndexCount])
select ''[?]'' + ''.''
+ quotename(sch.name) + ''.''
+ quotename(obj.name) AS DbObjectName,
COUNT(*) AS IndexCount
from [?].sys.objects obj 
inner join [?].sys.schemas sch on obj.schema_id = sch.schema_id
INNER JOIN [?].sys.indexes ix ON obj.object_id = ix.object_id
WHERE ix.index_id > 1
AND obj.schema_id <> 4
group by ''[?]'' + ''.''
+ quotename(sch.name) + ''.''
+ quotename(obj.name);

INSERT INTO #TableCounts([DatabaseName],[QualifiedObjectName],[TheCount])
SELECT  
''[?]'' AS DatabaseName,
QUOTENAME(SCHEMA_NAME(o.schema_id))  COLLATE SQL_Latin1_General_CP1_CI_AS 
+ ''.''  COLLATE SQL_Latin1_General_CP1_CI_AS  
+ QUOTENAME(o.name)  COLLATE SQL_Latin1_General_CP1_CI_AS  AS QualifiedObjectName,
SUM(ps.rows) AS TheCount
FROM   [?].sys.indexes AS i
INNER JOIN [?].sys.objects AS o
ON i.OBJECT_ID = o.OBJECT_ID
INNER JOIN [?].sys.[partitions] AS ps
ON i.OBJECT_ID = ps.OBJECT_ID
  AND i.index_id = ps.index_id
WHERE  i.index_id < 2
AND o.is_ms_shipped = 0
GROUP BY 
SCHEMA_NAME(o.schema_id),
o.name; 
'


;WITH IndexInfo
AS
(
SELECT idx.*,
fn.value As ColumnName
FROM sys.dm_db_missing_index_details idx
   CROSS APPLY STRING_SPLIT(REPLACE(REPLACE(REPLACE(ISNULL(idx.equality_columns,'') + ISNULL(',' + idx.inequality_columns,'') + ',' + ISNULL(idx.included_columns,''), '[', ''), ']', ''), ', ', ','), ',') AS fn
)
INSERT INTO #IndexSizes
select 
colz.[DatabaseName],
colz.[QualifiedObjectName],
idxz.equality_columns,
idxz.inequality_columns,
idxz.included_columns,
SUM(colz.[ApproxColumnSize]) * MAX(tabz.TheCount) /(1024 * 1024.0) AS Approximate_IndexSizeGB
from  IndexInfo idxz
INNER JOIN #ColumnSizes colz ON idxz.statement = (colz.DatabaseName + '.' + colz.QualifiedObjectName) -- AND idxz.Columnname = colz.ColumnName
INNER JOIN #TableCounts tabz ON colz.DatabaseName = tabz.DatabaseName AND colz.QualifiedObjectName = tabz.QualifiedObjectName

GROUP BY colz.[DatabaseName],
colz.[QualifiedObjectName],
idxz.equality_columns,
idxz.inequality_columns,
idxz.included_columns
--SELECT * FROM #TempIndexes


  DECLARE @FileGroup VARCHAR(128);
  DECLARE @ServerVersionNumber DECIMAL(5,2) = 0 -- .8,9.00,10.00,10.50,11.00, etc
  DECLARE @IndexOptions VARCHAR(256);
  DECLARE @OnlinePossible VARCHAR(5);
-----------------------------
--get version info
-----------------------------
  SELECT @ServerVersionNumber = CONVERT(DECIMAL(5,2),PARSENAME(CONVERT(VARCHAR(30), ISNULL(SERVERPROPERTY('productversion'),'')), 4)
                                                           + '.'
                                                           + Parsename(CONVERT(VARCHAR(30), ISNULL(Serverproperty('productversion'),'')), 3))

-----------------------------
--Enterprise?
-----------------------------
   SELECT @OnlinePossible =CASE 
        WHEN CONVERT(VARCHAR(128),SERVERPROPERTY('edition')) LIKE '%Enterprise%' --online is enterprise only
        THEN 'ON'
       
        ELSE 'OFF'
      END
  ;WITH MyFileGroups 
  AS
  (
  SELECT ROW_NUMBER() OVER(ORDER BY CASE WHEN name LIKE '%index%' THEN 1 ELSE 2 END, data_space_id DESC) AS RW,* FROM sys.filegroups WHERE type_desc='ROWS_FILEGROUP'
  )
    SELECT @FileGroup = name FROM MyFileGroups WHERE RW = 1

  SELECT @IndexOptions = 
    '        WITH (MAXDOP = 0, PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, '
    + CASE 
        WHEN @ServerVersionNumber < 10 --prior to SQL2008, not an option
        THEN ''
        WHEN @ServerVersionNumber < 11 --prior to SQL2012, sort in tempdb not much of a performance boost
        THEN ' SORT_IN_TEMPDB = OFF, ONLINE = ' + @OnlinePossible + ','  
        --2012 and above, significant performance boost.
        ELSE ' SORT_IN_TEMPDB = ON, ONLINE = ' + @OnlinePossible + ','  
      END
    + ' IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON '
    + CASE 
        WHEN value = 0 
        THEN '' 
        ELSE ', FILLFACTOR = ' + CONVERT(VARCHAR(4),value) 
      END + ',DATA_COMPRESSION=PAGE)' 
    + CHAR(13) + CHAR(10) 
  FROM sys.configurations 
  WHERE name='fill factor (%)';

  ;WITH ServerInfo
  AS
  (
        SELECT SQLServerRestartedDate = CONVERT(varchar(30),dbz.create_date,120)  from sys.databases dbz where dbz.name='tempdb'
  )
  ,LowellIndexes
  AS
  (
    select name from sys.indexes WHERE name LIKE 'IX_VI_%'
  )
     SELECT TOP (100) 
       ServerInfo.SQLServerRestartedDate,
       CONVERT(VARCHAR(128),@@serverName) AS ServerName,
      IndexAlreadyCreated =  (SELECT 1 FROM sys.indexes WHERE name = 'IX_VI_' + REPLACE(REPLACE(REPLACE(REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) -1)),'],[','_'),'[',''),']','')  + '_'
        + REPLACE(REPLACE(REPLACE(ISNULL(d.equality_columns,''),'], [','_'),'[',''),']','') 
        + REPLACE(REPLACE(REPLACE(ISNULL('_' + d.inequality_columns,''),'], [','_'),'[',''),']','') 
        +  CASE WHEN d.included_columns IS NOT NULL THEN '_Includes' ELSE '' END) ,
       CONVERT(VARCHAR(128),LEFT(d.statement,CHARINDEX('.',d.statement) -1) ) AS DatabaseName,
	   QualifiedTableName = d.statement,
       Tbl = REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) -1)),
       sz.Approximate_IndexSizeGB,
       [WeightedTotalCost] = FORMAT(s.avg_total_user_cost * s.avg_user_impact * (s.user_seeks + s.user_scans),'###,###,###,###,###') ,
       s.avg_total_user_cost,
       s.avg_user_impact, -- Query cost would reduce by this amount, on average.
       FORMAT(s.user_seeks,'###,###,###,###,###') AS user_seeks, 
       FORMAT(s.user_scans,'###,###,###,###,###') AS user_scans, 
       ISNULL(tix.IndexCount,0) As TableIndexCount,
       FORMAT(ISNULL(tct.[TheCount],0),'###,###,###,###,###') AS  TableTotalRows,
	   IndexComment = '-- ' 
        + d.statement 
        +  '[Total Cost]:' 
        + FORMAT(ISNULL(s.avg_total_user_cost * s.avg_user_impact * (s.user_seeks + s.user_scans),0),'###,###,###,###,###.00') 
        + '; avg_user_impact:' 
        +  CONVERT(VARCHAR(30),s.avg_user_impact ) 
        + '; user_seeks:' 
        +  FORMAT(s.user_seeks ,'###,###,###,###,###')
        + '; NC indexes:' 
        +  CONVERT(VARCHAR(30),ISNULL(tix.IndexCount,0)) 
        + '; TableTotalRows:' 
        +  FORMAT(ISNULL(tct.[TheCount],0),'###,###,###,###,###')
        + ';Approximate_IndexSizeGB:' + CONVERT(VARCHAR(30),sz.Approximate_IndexSizeGB) + 'GB'
        + '; Days Since Service Started:' + CONVERT(VARCHAR(10),DATEDIFF(dd,ServerInfo.SQLServerRestartedDate,GETDATE())) + ';' + CHAR(13)+ CHAR(10),
       --DatabaseName = LEFT(d.statement,CHARINDEX('.',d.statement) -1),
       [EqualityUsage] = d.equality_columns, 
       [InequalityUsage] = d.inequality_columns,
       [Include Columns] = d.included_columns,
       --building the command
       '-- USE ' + LEFT(d.statement,CHARINDEX('.',d.statement) -1) 
       + CHAR(13) + CHAR(10) 
       +'GO'
        + CHAR(13) + CHAR(10) 
        --comment block with stats
        + '-- ' 
        + d.statement 
        +  '[Total Cost]:' 
        + FORMAT(ISNULL(s.avg_total_user_cost * s.avg_user_impact * (s.user_seeks + s.user_scans),0),'###,###,###,###,###.00') 
        + ';Approximate_IndexSizeGB:' + CONVERT(VARCHAR(30),sz.Approximate_IndexSizeGB) + 'GB'
        + '; avg_user_impact:' 
        +  CONVERT(VARCHAR(30),s.avg_user_impact ) 
        + '; user_seeks:' 
        +  FORMAT(s.user_seeks ,'###,###,###,###,###')
        + '; NC indexes:' 
        +  CONVERT(VARCHAR(30),ISNULL(tix.IndexCount,0)) 
        + '; TableTotalRows:' 
        +  FORMAT(ISNULL(tct.[TheCount],0),'###,###,###,###,###')
        + '; Days Since Service Started:' + CONVERT(VARCHAR(10),DATEDIFF(dd,ServerInfo.SQLServerRestartedDate,GETDATE())) + ';' + CHAR(13)+ CHAR(10)
        --if not exists
        + 'IF EXISTS(SELECT OBJECT_ID(''' + d.statement + ''') )' + CHAR(13) + CHAR(10)
        + ' AND NOT EXISTS(SELECT 1 FROM [sys].[indexes] si INNER JOIN [sys].[objects] so ON so.[object_id] = si.[object_id] WHERE si.[Name] = N''' 
		+ 'IX_VI_' + REPLACE(REPLACE(REPLACE(REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) -1)),'],[','_'),'[',''),']','')  + '_'
        + REPLACE(REPLACE(REPLACE(ISNULL(d.equality_columns,''),'], [','_'),'[',''),']','') 
        + REPLACE(REPLACE(REPLACE(ISNULL('_' + d.inequality_columns,''),'], [','_'),'[',''),']','') 
        +  CASE WHEN d.included_columns IS NOT NULL THEN '_Includes' ELSE '' END + CASE WHEN  liz.name  IS NOT NULL THEN '2' ELSE '' END 
        + ''' AND so.[object_id] = OBJECT_ID(''' + d.statement + '''))' + CHAR(13) + CHAR(10) 
        --create index
        + '    CREATE NONCLUSTERED INDEX [IX_VI_' + REPLACE(REPLACE(REPLACE(REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) )),'],[','_'),'[',''),']','')  + '_'
        + REPLACE(REPLACE(REPLACE(ISNULL(d.equality_columns,''),'], [','_'),'[',''),']','') 
        + REPLACE(REPLACE(REPLACE(ISNULL('_' + d.inequality_columns,''),'], [','_'),'[',''),']','') 
        +  CASE WHEN d.included_columns IS NOT NULL THEN '_Includes' ELSE '' END + CASE WHEN  liz.name  IS NOT NULL THEN '2' ELSE '' END
        +'] '+ CHAR(13) + CHAR(10) 
        --on tablename
        + '      ON ' + d.statement + '(' + ISNULL(d.equality_columns,'') 
        + ISNULL( CASE WHEN d.equality_columns IS NULL THEN '' ELSE ',' END  + d.inequality_columns,'') + ')' 
        + CASE WHEN d.included_columns IS NOT NULL THEN ' INCLUDE(' + d.included_columns + ')' ELSE '' END + CHAR(13) + CHAR(10) 
        --indexoptions
        + @IndexOptions
        --filegroup
        + '      ON ' + QUOTENAME(@FileGroup) + ';'
         + CHAR(13) + CHAR(10) 
       +'GO'
        + CHAR(13) + CHAR(10) AS Indexcmd

      FROM sys.dm_db_missing_index_groups g 
        INNER JOIN sys.dm_db_missing_index_group_stats s 
          ON s.group_handle = g.index_group_handle 
        INNER JOIN sys.dm_db_missing_index_details d 
          ON d.index_handle = g.index_handle
          LEFT JOIN #TempIndexes tix ON d.statement = tix.DbObjectName
          LEFT JOIN #TableCounts tct ON d.statement = tct.DatabaseName + '.' + tct.[QualifiedObjectName]
          LEFT JOIN #IndexSizes sz ON  d.statement = sz.DatabaseName + '.' +  sz.QualifiedObjectName 
                                   AND ISNULL(sz.equality_columns,'') = ISNULL(d.equality_columns,'')
                                   AND ISNULL(sz.inequality_columns,'') = ISNULL(d.inequality_columns,'')
                                   AND ISNULL(sz.included_columns,'') = ISNULL(d.included_columns,'')
          LEFT JOIN LowellIndexes liz ON liz.name = 'IX_VI_' + REPLACE(REPLACE(REPLACE(REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) -1)),'],[','_'),'[',''),']','')  + '_'
        + REPLACE(REPLACE(REPLACE(ISNULL(d.equality_columns,''),'], [','_'),'[',''),']','') 
        + REPLACE(REPLACE(REPLACE(ISNULL('_' + d.inequality_columns,''),'], [','_'),'[',''),']','') 
        +  CASE WHEN d.included_columns IS NOT NULL THEN '_Includes' ELSE '' END
      CROSS JOIN ServerInfo
      WHERE 1=1
      AND (LEFT(d.statement,CHARINDEX('.',d.statement) -1) = quotename(db_name()) OR db_name()='master')
      --IGNORE database snapshots
      AND NOT EXISTS(SELECT * from sys.databases where source_database_id IS NOT NULL AND LEFT(d.statement,CHARINDEX('.',d.statement) -1) = QUOTENAME(name) )
      AND (REVERSE(LEFT(REVERSE(d.statement),CHARINDEX('.',REVERSE(d.statement)) -1)) = QUOTENAME(@TableNameToCheck) OR NULLIF(@TableNameToCheck,'') IS NULL)
      AND ISNULL(tct.[TheCount],0) > 0
      --AND ROUND(s.avg_total_user_cost * s.avg_user_impact * (s.user_seeks + s.user_scans),0) > 100000
      AND [s].[avg_total_user_cost] > 50
      --AND [s].[avg_user_impact] > 80
      AND [s].[user_seeks] > 10000
      ORDER BY [WeightedTotalCost] DESC;

IF @TableNameToCheck > ''
BEGIN
--desc: unused or little used indexes, modified to accommodate managed instances
  DECLARE @LastRestarted DATETIME;
  SELECT @LastRestarted = create_date FROM sys.databases WHERE name='tempdb';

      SELECT TOP (100) @LastRestarted AS ServerRestartedDate,
                  CASE WHEN i.name LIKE 'IX_VI_%' THEN 'IX_VI_' ELSE '' END AS DBAIndex,
	                SCHEMA_NAME(o.schema_id) AS SchemaName,
                    o.name AS ObjectName,
                    o.type_desc AS ObjectType,
                    i.name AS IndexName,
					 '--TotalRows: ' + CASE WHEN p.TableRows IS NULL THEN ' Disabled Index ' ELSE FORMAT(ISNULL(p.TableRows,1),'###,###,###,###,###') END
                    + ';ServerRestarted: ' + CONVERT(VARCHAR(5),DATEDIFF(dd,@LastRestarted,GETDATE())) + ' days prior on ' + CONVERT(VARCHAR(30),@LastRestarted,120)
                    + ';UserSeeks: ' + FORMAT(ISNULL(dm_ius.user_seeks ,0),'###,###,###,###,###')
                    + '|UserScans: ' + FORMAT(ISNULL(dm_ius.user_scans  ,0),'###,###,###,###,###')
                    + '|UserLookups: ' + FORMAT(ISNULL(dm_ius.user_lookups  ,0),'###,###,###,###,###')
                    + '|UserUpdates: ' + FORMAT(ISNULL(dm_ius.user_updates ,0),'###,###,###,###,###') + ';' As IndexNotes,
                    i.index_id AS IndexID,
                    FORMAT(dm_ius.user_seeks,'###,###,###,###,###')   AS UserSeek,
                    FORMAT(dm_ius.user_scans,'###,###,###,###,###')   AS UserScans,
                    FORMAT(dm_ius.user_lookups,'###,###,###,###,###') AS UserLookups,
                    FORMAT(dm_ius.user_updates,'###,###,###,###,###') AS UserUpdates,
                    FORMAT(p.TableRows,'###,###,###,###,###')         AS TableRows,
                    ISNULL([Index_Columns].[index_columns_key], '---') AS [index_columns_key] ,
                    ISNULL([Index_Columns].[index_columns_include], '---') AS [index_columns_include] ,
                    ISNULL(' WHERE (' + [i].[filter_definition] + ')', '') AS [index_where_statement] ,
                    '--TotalRows: ' + CASE WHEN p.TableRows IS NULL THEN ' Disabled Index? ' ELSE CONVERT(VARCHAR(30),ISNULL(p.TableRows,1)) END
                    + ';ServerRestarted: ' + CONVERT(VARCHAR(5),DATEDIFF(dd,@LastRestarted,GETDATE())) + ' days prior on ' + CONVERT(VARCHAR(30),@LastRestarted,120)
                    + ';UserSeeks: ' + FORMAT(ISNULL(dm_ius.user_seeks ,0),'###,###,###,###,###')
                    + '|UserScans: ' + FORMAT(ISNULL(dm_ius.user_scans  ,0),'###,###,###,###,###')
                    + '|UserLookups: ' + FORMAT(ISNULL(dm_ius.user_lookups  ,0),'###,###,###,###,###')
                    + '|UserUpdates: ' + FORMAT(ISNULL(dm_ius.user_updates ,0),'###,###,###,###,###') + ';' + CHAR(13) + CHAR(10)
                    +'  IF EXISTS(SELECT * from [sys].[indexes] WHERE name = ''' + i.name + ''' AND [object_id] = OBJECT_ID('''+ QUOTENAME(SCHEMA_NAME(o.schema_id)) + '.' + QUOTENAME(o.name) +'''))'+ CHAR(13) + CHAR(10)
                    + '    DROP INDEX ' + QUOTENAME(i.name) + ' ON '
                    + QUOTENAME(OBJECT_SCHEMA_NAME(i.object_id)) + '.' 
                    + QUOTENAME(OBJECT_NAME(i.object_id)) AS 'drop statement',
					'IF NOT EXISTS(SELECT * FROM [sys].[indexes] WHERE [name] = ''' + [i].[name] COLLATE SQL_Latin1_General_CP1_CI_AS 
					+ ''' AND object_schema_name([object_id]) = ''' + [s].[name]  COLLATE SQL_Latin1_General_CP1_CI_AS
                    + ''' and object_name([object_id]) = ''' + [o].[name]  COLLATE SQL_Latin1_General_CP1_CI_AS + ''') CREATE ' 
					+ CASE 
					    WHEN [i].[is_unique] = 1 
						THEN ' UNIQUE '
                        ELSE ''
                      END 
					+ CASE 
					    WHEN [i].[index_id] = 1 
					    THEN ' CLUSTERED '
                        ELSE ' NONCLUSTERED  '
                      END 
					  + 'INDEX ' + QUOTENAME([i].[name]) + ' ON '
                      + QUOTENAME([s].[name]) + '.' + QUOTENAME([o].[name]) 
					  + ISNULL(' (' + [Index_Columns].[index_columns_key] + ') ','')
                      + CASE WHEN [Index_Columns].[index_columns_include] IS NULL OR [Index_Columns].[index_columns_include] = '---' THEN ''
                             ELSE ' INCLUDE(' + [Index_Columns].[index_columns_include] + ')'
                        END +  ISNULL(' WHERE (' + [i].[filter_definition] + ')', '')  
						+ ' WITH ( DATA_COMPRESSION = ' + ISNULL([p].data_compression_desc,'PAGE') + ' )'
						AS [cmd]
      FROM   sys.indexes i 
             INNER JOIN sys.objects o
                     ON i.object_id = o.object_id
             INNER JOIN sys.schemas s
                     ON o.schema_id = s.schema_id
             LEFT JOIN sys.dm_db_index_usage_stats dm_ius
                     ON i.index_id = dm_ius.index_id
                        AND dm_ius.object_id = i.object_id
						AND dm_ius.database_id = DB_ID()
             LEFT JOIN (SELECT SUM(p.rows) TableRows,
                                p.index_id,
                                p.object_id,
								p.data_compression_desc
                         FROM   sys.partitions p
                         GROUP  BY p.index_id,
                                   p.object_id,
								   p.data_compression_desc) p
                     ON p.index_id = dm_ius.index_id
                        AND dm_ius.object_id = p.object_id
             OUTER APPLY ( SELECT  LEFT([Index_Columns].[index_columns_key], LEN([Index_Columns].[index_columns_key]) - 1) AS [index_columns_key] ,
                                          LEFT([Index_Columns].[index_columns_include], LEN([Index_Columns].[index_columns_include]) - 1) AS [index_columns_include] ,
                                          LEFT([Index_Columns].[index_columns_where], LEN([Index_Columns].[index_columns_where]) - 1) AS [index_columns_where]
                                  FROM    ( SELECT  ( SELECT  QUOTENAME([COLS].[name]) + CASE WHEN [IXCOLS].[is_descending_key] = 0 THEN ' asc'
                                                                                          ELSE ' desc'
                                                                                     END + ',' + ' '
                                                      FROM    [sys].[index_columns] [IXCOLS]
                                                              INNER JOIN [sys].[columns] [COLS] ON [IXCOLS].[column_id] = [COLS].[column_id]
                                                                                             AND [IXCOLS].[object_id] = [COLS].[object_id]
                                                      WHERE   [IXCOLS].[is_included_column] = 0
                                                              AND [i].[object_id] = [IXCOLS].[object_id]
                                                              AND [i].[index_id] = [IXCOLS].[index_id]
                                                      ORDER BY [IXCOLS].[key_ordinal]
                                                    FOR
                                                      XML PATH('')
                                                    ) AS [index_columns_key] ,
                                                    ( SELECT  QUOTENAME([COLS].[name]) + ',' + ' '
                                                      FROM    [sys].[index_columns] [IXCOLS]
                                                              INNER JOIN [sys].[columns] [COLS] ON [IXCOLS].[column_id] = [COLS].[column_id]
                                                                                             AND [IXCOLS].[object_id] = [COLS].[object_id]
                                                      WHERE   [IXCOLS].[is_included_column] = 1
                                                              AND [i].[object_id] = [IXCOLS].[object_id]
                                                              AND [i].[index_id] = [IXCOLS].[index_id]
                                                      ORDER BY [IXCOLS].[index_column_id]
                                                    FOR
                                                      XML PATH('')
                                                    ) AS [index_columns_include] ,
                                                    ( SELECT  QUOTENAME([COLS].[name]) + ',' + ' '
                                                      FROM    [sys].[index_columns] [IXCOLS]
                                                              INNER JOIN [sys].[columns] [COLS] ON [IXCOLS].[column_id] = [COLS].[column_id]
                                                                                             AND [IXCOLS].[object_id] = [COLS].[object_id]
                                                      WHERE   [IXCOLS].[is_included_column] = 1
                                                              AND [i].[object_id] = [IXCOLS].[object_id]
                                                              AND [i].[index_id] = [IXCOLS].[index_id]
                                                      ORDER BY [IXCOLS].[index_column_id]
                                                    FOR
                                                      XML PATH('')
                                                    ) AS [index_columns_where]
                                          ) AS [Index_Columns]
                                ) AS [Index_Columns]
      WHERE  OBJECTPROPERTY(i.object_id, 'IsUserTable') = 1

             AND i.type_desc = 'nonclustered'
            -- AND i.is_primary_key = 0
           --  AND i.is_unique_constraint = 0
             AND(@TableNameToCheck ='' OR o.name = @TableNameToCheck)
      ORDER  BY ( dm_ius.user_seeks + dm_ius.user_scans
                  + dm_ius.user_lookups ) ASC

END