 --desc: builds compression statements for all tables and indexes;same as envcompress, except removing db references, adding rowcount.
 IF EXISTS(SELECT * FROM sys.databases WHERE database_id = DB_ID() AND is_read_only = 1)
 BEGIN
    SELECT 'Database' + CONVERT(VARCHAR(128),DB_NAME()) + ' is Read Only, Skipping' AS Notes
 END
 ELSE
 BEGIN
 DECLARE @UserIndexCount INT
 SELECT @UserIndexCount = COUNT(*) FROM sys.indexes WHERE OBJECT_SCHEMA_NAME(object_id) <> 'sys'
 PRINT @UserIndexCount
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
;WITH DatabaseSettings 
AS
(
SELECT 
    CONVERT(BIGINT,-10) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--Changing RECOVERY on PRODUCTION IS NOT RECOMMENDED and is thus commented out, but left as an option.') AS CompressionCommand 
FROM sys.databases WHERE name = DB_NAME() AND [recovery_model_desc] = 'FULL' UNION ALL
SELECT 
    CONVERT(BIGINT,-9) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--ALTER DATABASE ' + QUOTENAME(DB_NAME()) + ' SET RECOVERY BULK_LOGGED;') AS CompressionCommand 
FROM sys.databases WHERE name = DB_NAME() AND [recovery_model_desc] = 'FULL' UNION ALL
SELECT  
    CONVERT(BIGINT,-10) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--Database is In Simple Recovery, no toggle for BULK_LOGGED;') AS CompressionCommand 
FROM sys.databases WHERE name = DB_NAME() AND [recovery_model_desc] = 'SIMPLE' 
UNION ALL
SELECT 
 CONVERT(BIGINT,-4) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    ISNULL(CONVERT(NVARCHAR(4000),'--Cannot Compress: ' + CONVERT(VARCHAR(30),SUM(CASE WHEN data_compression_desc = 'None' THEN 1 ELSE 0 END)) + ' out of ' + CONVERT(VARCHAR(30),@UserIndexCount) + ' External Tables;'),'--No External Tables') AS CompressionCommand 
FROM   sys.indexes AS i
            INNER JOIN sys.tables AS o
                    ON i.object_id = o.object_id AND o.is_external = 1
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id 
WHERE schema_id <> 4
UNION ALL
SELECT 
 CONVERT(BIGINT,-3) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--No Compression: ' + CONVERT(VARCHAR(30),SUM(CASE WHEN data_compression_desc = 'None' THEN 1 ELSE 0 END)) + ' out of ' + CONVERT(VARCHAR(30),@UserIndexCount) + ' Indexes featuring No Compression;') AS CompressionCommand 
FROM   sys.indexes AS i
            INNER JOIN sys.tables AS o
                    ON i.object_id = o.object_id AND o.is_external = 0
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id 
WHERE schema_id <> 4
UNION ALL
SELECT 
 CONVERT(BIGINT,-2) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--Page Compression: ' + CONVERT(VARCHAR(30),SUM(CASE WHEN data_compression_desc = 'Page' THEN 1 ELSE 0 END)) + ' out of ' + CONVERT(VARCHAR(30),@UserIndexCount) + ' Indexes featuring Page Compression;') AS CompressionCommand 
FROM   sys.indexes AS i
            INNER JOIN sys.objects AS o
                    ON i.object_id = o.object_id
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id 
WHERE schema_id <> 4
UNION ALL

SELECT 
 CONVERT(BIGINT,-1) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--Row Compression: ' + CONVERT(VARCHAR(30),SUM(CASE WHEN data_compression_desc = 'Row' THEN 1 ELSE 0 END)) + ' out of ' + CONVERT(VARCHAR(30),@UserIndexCount) + ' Indexes featuring Row Compression;') AS CompressionCommand 
FROM   sys.indexes AS i
            INNER JOIN sys.objects AS o
                    ON i.object_id = o.object_id
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id 
WHERE schema_id <> 4
UNION ALL
SELECT 
    CONVERT(BIGINT,2147483647) AS TheCount,
    CONVERT(NVARCHAR(517),NULL) AS [QualifiedObjectName], 
    CONVERT(NVARCHAR(128),NULL) AS [SchemaName],
    CONVERT(sysname,NULL) AS [TableName],
    CONVERT(sysname,NULL) AS [name],
    CONVERT(INT,NULL) AS [index_id],
    CONVERT(NVARCHAR(60),NULL) AS [data_compression_desc],
    CONVERT(NVARCHAR(4000),'--ALTER DATABASE ' + QUOTENAME(DB_NAME()) + ' SET FULL;') AS CompressionCommand 
FROM sys.databases WHERE name = DB_NAME() AND [recovery_model_desc] = 'FULL' 
)
, T1
AS
(
     SELECT  
       QUOTENAME(SCHEMA_NAME(o.schema_id))  COLLATE SQL_Latin1_General_CP1_CI_AS 
       + '.'  COLLATE SQL_Latin1_General_CP1_CI_AS  
       + QUOTENAME(o.name)  COLLATE SQL_Latin1_General_CP1_CI_AS  AS QualifiedObjectName,
     SCHEMA_NAME(o.schema_id) AS SchemaName,
            o.name AS ObjectName,
            SUM(ps.rows) AS TheCount,
            'SELECT * FROM '  COLLATE SQL_Latin1_General_CP1_CI_AS 
            + QUOTENAME(SCHEMA_NAME(o.schema_id))  COLLATE SQL_Latin1_General_CP1_CI_AS 
            + '.' COLLATE SQL_Latin1_General_CP1_CI_AS 
            + QUOTENAME(o.name)  COLLATE SQL_Latin1_General_CP1_CI_AS  AS cmd,
            MAX(ps.data_compression_desc) AS data_compression_desc
     FROM   sys.indexes AS i
            INNER JOIN sys.objects AS o
                    ON i.object_id = o.object_id
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id
     WHERE  i.index_id < 2
           -- AND o.is_ms_shipped = 0
     GROUP BY 
       SCHEMA_NAME(o.schema_id),
       o.name 
)
,NonCompressedIndexes
AS
(
SELECT COUNT(*) AS IndexCount FROM   sys.indexes AS i
            INNER JOIN sys.objects AS o
                    ON i.object_id = o.object_id
            INNER JOIN sys.[partitions] AS ps
                    ON i.object_id = ps.object_id
                       AND i.index_id = ps.index_id 
            WHERE data_compression_desc = 'NONE'
)
,Commands
AS
(
SELECT 
T1.[TheCount],
QUOTENAME(SCHEMA_NAME(objz.schema_id)) + '.' + QUOTENAME(objz.name) AS QualifiedObjectName,
SCHEMA_NAME(objz.schema_id) AS SchemaName,
objz.name AS TableName, 
ISNULL(IX.name,'{heap}') As IndexName, 
IX.index_id,
--objz.object_id, 
--sp.partition_id, 
--sp.partition_number, 
--sp.data_compression,
SP.data_compression_desc,
'RAISERROR(''TotalRows: ' + CONVERT(VARCHAR(30),TheCount) + '; Item ' 
+ CONVERT(VARCHAR(30),ROW_NUMBER() OVER (ORDER BY T1.[TheCount] ,SP.data_compression_desc,objz.name)) + ' of ' + CONVERT(VARCHAR(30),@UserIndexCount) + '; ' +
CASE 
  WHEN SP.data_compression_desc = 'NONE'
  THEN CASE
       WHEN IX.index_id= 0
       THEN 'Rebuilding Table  '+ CONVERT(VARCHAR(30),IX.index_id) + ' - ' + QUOTENAME(OBJECT_SCHEMA_NAME([objz].[object_id])) 
            + '.' + QUOTENAME(OBJECT_NAME([objz].[object_id])) + ''',0,1) WITH NOWAIT;' + CHAR(13) + CHAR(10)
            + 'ALTER TABLE ' 
            + QUOTENAME(OBJECT_SCHEMA_NAME([objz].[object_id])) 
            + '.' + QUOTENAME(OBJECT_NAME([objz].[object_id])) 
            + ' REBUILD WITH (MAXDOP = 0,data_compression = PAGE) ;' + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10)
       ELSE 'Rebuilding Index '+ IX.name + ' - ' + QUOTENAME(OBJECT_SCHEMA_NAME([objz].[object_id])) 
            + '.' + QUOTENAME(OBJECT_NAME([objz].[object_id])) + ''',0,1) WITH NOWAIT;' + CHAR(13) + CHAR(10)
             + ' -- ALTER INDEX ' +  QUOTENAME(IX.[name]) 
            + ' ON ' 
            + QUOTENAME(OBJECT_SCHEMA_NAME([objz].[object_id])) 
            + '.' + QUOTENAME(OBJECT_NAME([objz].[object_id])) 
            + ' PAUSE;  --or RESUME'
            + 'ALTER INDEX ' 
            + QUOTENAME(IX.[name]) 
            + ' ON ' 
            + QUOTENAME(OBJECT_SCHEMA_NAME([objz].[object_id])) 
            + '.' + QUOTENAME(OBJECT_NAME([objz].[object_id])) 
            + ' REBUILD WITH (MAXDOP = 0, DATA_COMPRESSION = PAGE,ONLINE = ON,RESUMABLE = ON ) ;' + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10)
       END
  ELSE ''
END AS [CompressionCommand] 
FROM sys.partitions SP  
INNER JOIN sys.tables objz   ON objz.object_id = SP.object_id  AND objz.[is_external] = 0 
INNER JOIN T1   ON T1.QualifiedObjectName = QUOTENAME(SCHEMA_NAME(objz.schema_id)) + '.' + QUOTENAME(objz.name) 
LEFT OUTER JOIN sys.indexes IX   ON SP.object_id = IX.object_id AND SP.index_id = IX.index_id
CROSS JOIN NonCompressedIndexes recz

)
SELECT * FROM Commands WHERE 1=1
AND data_compression_desc = 'NONE'
UNION ALL SELECT * FROM DatabaseSettings
ORDER BY [TheCount] ,data_compression_desc,TableName
END
