USE [master];
GO
--#################################################################################################
-- Remove Previous Procs,Events and jobs
--#################################################################################################
-- delete all backup/restore history prior to a specified date
-- delete all backup/restore history prior to a specified date
--DECLARE @dt DATETIME =DATEADD(dd,-30,GETDATE());
--EXECUTE msdb.dbo.sysmail_delete_mailitems_sp @sent_before = @dt;
--SET @dt = DATEADD(dd,-30,GETDATE());
--EXECUTE msdb.dbo.sp_purge_jobhistory @oldest_date = @dt;
--EXECUTE msdb.dbo.sp_delete_backuphistory @dt;
--EXECUTE  msdb.dbo.sysmail_delete_log_sp @logged_before = @dt;
--USE msdb;ALTER INDEX [sysmail_mailitems_id_MustBeUnique]  ON dbo.sysmail_mailitems REBUILD;

GO
IF EXISTS ( SELECT * FROM sys.server_event_sessions WHERE name = 'ApplicationErrors' )
  DROP EVENT SESSION [ApplicationErrors] ON SERVER;
GO
USE [msdb];
GO
IF OBJECT_ID('[dbo].[ApplicationErrorLog]') IS NOT NULL 
DROP TABLE [dbo].[ApplicationErrorLog]; 
GO
IF OBJECT_ID('[dbo].[usp_ApplicationErrorLog]') IS NOT NULL 
DROP  PROCEDURE [dbo].[usp_ApplicationErrorLog]; 
GO
IF OBJECT_ID('[dbo].[usp_Cycle_ApplicationErrors]') IS NOT NULL 
DROP  PROCEDURE [dbo].[usp_Cycle_ApplicationErrors]; 
GO
IF  EXISTS (SELECT * FROM [msdb].[dbo].[sysjobs] WHERE name = N'[DBA]_ApplicationErrors')
  EXECUTE [msdb].[dbo].[sp_delete_job] @job_name=N'[DBA]_ApplicationErrors', @delete_unused_schedule=1;
GO
--this was renamed int he past, remove any old job also!
IF  EXISTS (SELECT * FROM [msdb].[dbo].[sysjobs] WHERE name = N'DBA ApplicationErrors')
  EXECUTE [msdb].[dbo].[sp_delete_job] @job_name=N'DBA ApplicationErrors', @delete_unused_schedule=1;
GO
--#################################################################################################
-- Create the extended event
--#################################################################################################
CREATE EVENT SESSION [ApplicationErrors] ON SERVER 
ADD EVENT sqlserver.error_reported 
    (
      ACTION (sqlserver.client_hostname, 
              sqlserver.client_app_name,
              sqlserver.database_id, 
              sqlserver.sql_text, 
              sqlserver.username,
              sqlserver.session_id 
              )
      WHERE ([severity] >= ( 11 )) 
    )
ADD TARGET package0.ring_buffer
WITH (MAX_DISPATCH_LATENCY = 1 SECONDS,STARTUP_STATE = ON);
GO
-- Alter the Event Session and Start it.
ALTER EVENT SESSION [ApplicationErrors]
ON SERVER
STATE=START;
GO
--#################################################################################################
-- create the supporting tables, procs and job
--#################################################################################################
IF OBJECT_ID('[msdb].[dbo].[ApplicationErrorLog]') IS NOT NULL 
DROP TABLE [dbo].[ApplicationErrorLog]; 
GO
CREATE TABLE [msdb].[dbo].[ApplicationErrorLog] ( 
[ApplicationErrorLogID]            BIGINT           IDENTITY(1,1)          NOT NULL,
[TS]            DATETIME                                    NULL,
[DBName]        VARCHAR(128)                                NULL,
[Err_No]        INT                                         NULL,
[Err_Severity]  INT                                         NULL,
[Err_Message]   NVARCHAR(MAX)                               NULL,
[UserID]        VARCHAR(128)                                NULL,
[spid]          INT                                         NULL,
[Hostname]      VARCHAR(128)                                NULL,
[DBCall]        NVARCHAR(MAX)                               NULL,
[AppName]       VARCHAR(128)                                NULL,
[Err_State]     INT                                         NULL,
[LastUpdated]   DATETIME                                    NULL  CONSTRAINT [DF__DBAError___ApplicationErrorLog__LastUpdated] DEFAULT (GETDATE()),
[Duplicates]    INT                                     NOT NULL  CONSTRAINT [DF__DBAError___ApplicationErrorLog__Duplicates] DEFAULT ((0)),
CONSTRAINT   [PK_ApplicationErrorLog]  PRIMARY KEY NONCLUSTERED    ([ApplicationErrorLogID] ASC));

CREATE CLUSTERED INDEX IX_ApplicationErrorLog_TS ON [msdb].[dbo].[ApplicationErrorLog](TS);
GO
--#################################################################################################

--#################################################################################################
-- base query:
--#################################################################################################
--SELECT 
--DATEADD(second,
--      DATEDIFF(second, SYSUTCDATETIME(), SYSDATETIME()) + 86,n.value('(@timestamp)[1]', 'datetime2')) AS [timestamp], 
--      DB_NAME(n.value('(./action[@name="database_id"]/value)[1]','int'))         AS DbName,
--      n.value('(./data[@name="error"]/value)[1]', 'int')                         AS ErrorNumber,
--      n.value('(./data[@name="severity"]/value)[1]', 'nvarchar(max)')            AS ErrorSeverity,
--      n.value('(./data[@name="message"]/value)[1]', 'nvarchar(max)')             AS ErrorMessage,
--      n.value('(./action[@name="username"]/value)[1]','nvarchar(max)')           AS UserId,
--      n.value('(./action[@name="session_id"]/value)[1]','nvarchar(max)')         AS SPID,
--      n.value('(./action[@name="client_hostname"]/value)[1]','nvarchar(max)')    AS Hostname,
--      n.value('(./action[@name="sql_text"]/value)[1]','nvarchar(max)')           AS DBCallParameters,
--      n.value('(./action[@name="client_app_name"]/value)[1]','nvarchar(max)')    AS AppName,
--      n.value('(./data[@name="state"]/value)[1]', 'int')                         AS ErrorState
--FROM 
--(SELECT
--    CAST(target_data AS XML) AS event_data
--FROM sys.dm_xe_session_targets xet
--JOIN sys.dm_xe_sessions xes
--    ON xes.address = xet.event_session_address
--    WHERE xes.name = 'ApplicationErrors'
--  AND xet.target_name = 'ring_buffer'
--) as tab
--CROSS APPLY event_data.nodes('RingBufferTarget/event') as q(n)
--#################################################################################################
GO
IF OBJECT_ID('[dbo].[usp_ApplicationErrorLog]') IS NOT NULL 
DROP  PROCEDURE [dbo].[usp_ApplicationErrorLog] 
GO
--#################################################################################################
-- PROCEDURE [dbo].[usp_ApplicationErrorLog]
--#################################################################################################
--updated 2020-07-29 to do all the upserts within this procedure, instead of as an inline step in the job.
--that result sin cleaner, more accurate counts
CREATE PROCEDURE [dbo].[usp_ApplicationErrorLog]
--WITH ENCRYPTION
AS
SET NOCOUNT ON;
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
BEGIN --PROC
IF OBJECT_ID('tempdb.[dbo].[#ApplicationErrorLog]') IS NOT NULL 
  DROP TABLE [dbo].[#ApplicationErrorLog] 

DECLARE @target_data XML;
SELECT 
  @target_data = CAST(target_data AS XML)
FROM sys.dm_xe_sessions AS s 
  INNER JOIN sys.dm_xe_session_targets AS t 
    ON t.event_session_address = s.address
WHERE s.name = N'ApplicationErrors'
  AND t.target_name = N'ring_buffer';

SELECT 
    DATEADD(SECOND,
    DATEDIFF(SECOND, SYSUTCDATETIME(), SYSDATETIME()) + 86,n.value('(@timestamp)[1]', 'datetime2')) AS [timestamp], 
    DB_NAME(n.value('(./action[@name="database_id"]/value)[1]','int'))         AS DbName,
    n.value('(./data[@name="error"]/value)[1]', 'int')                         AS ErrorNumber,
    n.value('(./data[@name="severity"]/value)[1]', 'nvarchar(max)')            AS ErrorSeverity,
    n.value('(./data[@name="message"]/value)[1]', 'nvarchar(max)')             AS ErrorMessage,
    n.value('(./action[@name="username"]/value)[1]','nvarchar(max)')           AS UserId,
    n.value('(./action[@name="session_id"]/value)[1]','nvarchar(max)')         AS SPID,
    n.value('(./action[@name="client_hostname"]/value)[1]','nvarchar(max)')    AS Hostname,
    n.value('(./action[@name="sql_text"]/value)[1]','nvarchar(max)')           AS DBCallParameters,
    n.value('(./action[@name="client_app_name"]/value)[1]','nvarchar(max)')    AS AppName,
    n.value('(./data[@name="state"]/value)[1]', 'int')                         AS ErrorState
INTO #ApplicationErrorLog
FROM @target_data.nodes('RingBufferTarget/event') AS q(n);

--#################################################################################################
-- insert all the results
--#################################################################################################
INSERT INTO msdb.dbo.ApplicationErrorLog 
      ([TS],[DBName],[Err_No],[Err_Severity],[Err_Message],[UserID],[spid],[Hostname],[DBCall],[AppName],[Err_State],[Duplicates])
SELECT [timestamp],[DbName],[ErrorNumber],[ErrorSeverity],[ErrorMessage],[UserId],[SPID],[Hostname],[DBCallParameters],[AppName],[ErrorState],1 AS [Duplicates]
FROM #ApplicationErrorLog

--#################################################################################################
-- fix our totals
--#################################################################################################
;WITH MyCTE
  AS
  (
  SELECT 
    DBName,
    Err_Severity,
    Err_Message,
    UserID,
    LEFT(DBCall,120) AS DBCall,
    SUM(Duplicates) AS OccurranceCount 
  FROM    [msdb].[dbo].[ApplicationErrorLog] appz
  GROUP BY DBName,Err_Severity,Err_Message,UserID,LEFT(DBCall,120)
  )
  UPDATE appz
  SET appz.Duplicates = MyCTE.[OccurranceCount]
  FROM [msdb].[dbo].[ApplicationErrorLog] appz
  INNER JOIN MyCTE 
    ON  [appz].DBName        = [MyCTE].[DBName]
    AND [appz].Err_Severity  = [MyCTE].[Err_Severity]
    AND [appz].Err_Message   = [MyCTE].[Err_Message]
    AND [appz].UserID        = [MyCTE].[UserID]
    AND LEFT([appz].DBCall,120) = LEFT([MyCTE].[DBCall],120)
  WHERE [appz].DBName        = [MyCTE].[DBName]
    AND [appz].Err_Severity  = [MyCTE].[Err_Severity]
    AND [appz].Err_Message   = [MyCTE].[Err_Message]
    AND [appz].UserID        = [MyCTE].[UserID]
    AND LEFT([appz].DBCall,120) = LEFT([MyCTE].[DBCall],120)

--#################################################################################################
-- remove duplicates to save space
--#################################################################################################
  ;WITH MyCTE
  AS
  (
  SELECT ROW_NUMBER() OVER(PARTITION BY DBName,Err_Severity,Err_Message,UserID,LEFT(DBCall,120) ORDER BY ApplicationErrorLogID) AS  RW,*
  FROM [msdb].[dbo].[ApplicationErrorLog] appz
  )
  DELETE
  --SELECT * 
  FROM MyCTE WHERE RW > 1;
--#################################################################################################
-- entries older than x days to also save space
--#################################################################################################
DELETE FROM [msdb].[dbo].[ApplicationErrorLog] WHERE [TS] < DATEADD(dd,-15,GETDATE());

END; --PROC

GO





ALTER AUTHORIZATION ON [dbo].[usp_ApplicationErrorLog] TO  SCHEMA OWNER; 
GO
USE [master];
GO

USE [msdb];
GO
IF OBJECT_ID('[dbo].[usp_Cycle_ApplicationErrors]') IS NOT NULL 
DROP  PROCEDURE [dbo].[usp_Cycle_ApplicationErrors]; 
GO
USE [msdb];
GO
--#################################################################################################
--  PROCEDURE [dbo].[usp_Cycle_ApplicationErrors]
--#################################################################################################
CREATE PROCEDURE [dbo].[usp_Cycle_ApplicationErrors]  @SetStatus VARCHAR(30) 
AS
BEGIN --PROC
DECLARE @ee VARCHAR(128) = 'ApplicationErrors';
 BEGIN
  IF @SetStatus = 'OFF'
    BEGIN
     --try it twice
      IF EXISTS (SELECT COUNT(name) FROM sys.dm_xe_sessions WHERE name = @ee GROUP BY name HAVING COUNT(name) = 1 )
        ALTER EVENT SESSION [ApplicationErrors] ON SERVER STATE = STOP;
        WAITFOR DELAY '00:00:01'
       IF EXISTS (SELECT COUNT(name) FROM sys.dm_xe_sessions WHERE name = @ee GROUP BY name HAVING COUNT(name) = 1 )
        ALTER EVENT SESSION [ApplicationErrors] ON SERVER STATE = STOP;
    END
   ELSE --@SetStatus = ON -turn it on
     BEGIN
       --try it twice
      IF NOT EXISTS (SELECT COUNT(name) FROM sys.dm_xe_sessions WHERE name = @ee GROUP BY name HAVING COUNT(name) = 1 )
        ALTER EVENT SESSION [ApplicationErrors] ON SERVER STATE = START;
        WAITFOR DELAY '00:00:01'
       IF NOT EXISTS (SELECT COUNT(name) FROM sys.dm_xe_sessions WHERE name = @ee GROUP BY name HAVING COUNT(name) = 1 )
        ALTER EVENT SESSION [ApplicationErrors] ON SERVER STATE = START;
     END
 END;
END; -- PROC

GO


IF  EXISTS (SELECT * FROM msdb.dbo.sysjobs_view WHERE name = N'DBA ApplicationErrors')
EXECUTE msdb.dbo.sp_delete_job @job_name=N'DBA ApplicationErrors', @delete_unused_schedule=1;
GO
IF  EXISTS (SELECT * FROM msdb.dbo.sysjobs_view WHERE name = N'ViscosityNA_DBA ApplicationErrors')
EXECUTE msdb.dbo.sp_delete_job @job_name=N'ViscosityNA_DBA ApplicationErrors', @delete_unused_schedule=1;
GO
--#################################################################################################
-- Create the job
--#################################################################################################
USE [msdb]
GO
DECLARE @saAccount varchar(128)
SELECT @saAccount = name from sys.server_principals WHERE sid = 0x01
PRINT 'sa account is ' + @saAccount
/****** Object:  Job [DBA_ApplicationErrors]    Script Date: 7/30/2020 8:03:42 AM ******/
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0
/****** Object:  JobCategory [Data Collector]    Script Date: 7/30/2020 8:03:42 AM ******/
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'Data Collector' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'Data Collector'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'[DBA]_ApplicationErrors', 
		@enabled=1, 
		@notify_level_eventlog=3, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Logs Event Errors into msdb.dbo.ApplicationErrorLog', 
		@category_name=N'Data Collector', 
		@owner_login_name=@saAccount, @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [LogEventErrorCapture]    Script Date: 7/30/2020 8:03:43 AM ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'LogEventErrorCapture', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'
      DELETE FROM msdb.dbo.ApplicationErrorLog   WITH (TABLOCKX) WHERE TS < DATEADD(dd,-15,GETDATE());
     
        EXEC [dbo].[usp_ApplicationErrorLog];
', 
		@database_name=N'msdb', 
		@flags=4
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [CycleApplicationErrors OFF]    Script Date: 7/30/2020 8:03:43 AM ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'CycleApplicationErrors OFF', 
		@step_id=2, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'EXEC [msdb].[dbo].[usp_Cycle_ApplicationErrors]  @SetStatus = ''OFF''', 
		@database_name=N'msdb', 
		@flags=4
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [CycleApplicationErrors ON]    Script Date: 7/30/2020 8:03:43 AM ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'CycleApplicationErrors ON', 
		@step_id=3, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'EXEC [msdb].[dbo].[usp_Cycle_ApplicationErrors]  @SetStatus = ''ON''', 
		@database_name=N'msdb', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'DBAApplicationErrors', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=4, 
		@freq_subday_interval=5, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20140110, 
		@active_end_date=99991231, 
		@active_start_time=0, 
		@active_end_time=235959, 
		@schedule_uid=N'a53a7018-0bc1-4975-976f-e0d55451c5b3'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
GO






